import {
  select,
  csv,
  scaleLinear,
  max,
  scalePoint,
  axisLeft,
  axisBottom,
  format
} from 'd3';

const svg = select('svg');
const width = +svg.attr('width');
const height = +svg.attr('height');

const render = data => {
  const yAxisLabel = 'Protocol';
  const xAxisLabel = 'Average Packet Length';
  const xValue = d => d.length;
  const yValue = d => d.protocol;
  const margin = { top: 50, right: 40, bottom: 77, left: 180 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  const xScale = scaleLinear()
    .domain([0, max(data, xValue)])
    .range([0, innerWidth])
    .nice();
  
  const yScale = scalePoint()
    .domain(data.map(yValue))
    .range([0, innerHeight])
    .padding(0.7);
  
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);
  
  const xAxisTickFormat = number =>
    format('.2s')(number).replace('0.0', '0');
  
  
  // Axis Modifications and Labels
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth);
  const yAxisG = g.append('g').call(yAxis)
  yAxisG.selectAll('.domain').remove();
  yAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', -85)
      .attr('x', -innerHeight / 2)
      .attr('text-anchor', 'middle')
      .attr('transform', `rotate(-90)`)
      .text(yAxisLabel);
  
  const xAxis = axisBottom(xScale)
    .tickFormat(xAxisTickFormat)
    .tickSize(-innerHeight);
  const xAxisG = g.append('g').call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  xAxisG.select('.domain').remove();
  xAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', 65)
      .attr('x', innerWidth / 2)
      .attr('fill', 'black')
      .text('Average Packet Length');
  
  g.append('text')
      .attr('class', 'title')
      .attr('y', -10)
      .attr('x', 100)
      .text(xAxisLabel);
  

  // Define the div for the tooltip
  var div = d3.select("body").append("div")	
      .attr("class", "tooltip")				
      .style("opacity", 0);
  
  // Insert Data
  g.selectAll('circle').data(data)
    .enter().append('circle')
      .attr('cy', d => yScale(yValue(d)))
      .attr('cx', d => xScale(xValue(d)))
      .attr('r', 12)
    .on("mouseover", function(d) {		
            div.transition()		
                .duration(100)		
                .style("opacity", .9);		
            div.html(d.length)	
                .style("left", (d3.event.pageX) + "px")		
                .style("top", (d3.event.pageY - 28) + "px");	
            })					
        .on("mouseout", function(d) {		
            div.transition()		
                .duration(500)		
                .style("opacity", 0);	
        });;
};

csv('data.csv').then(data => {
  data.forEach(d => {
    d.length = +d.length;
  });
  render(data);
});